@@ -23,6 +23,7 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 
+#include <algorithm>
 #include <optional>
 
 #include "libdiscover_backend_packagekit_debug.h"
@@ -190,8 +191,11 @@ public:
     QString longDescription() override
     {
         QStringList changes;
-        const auto resources = withoutDuplicates();
-        for (auto resource : resources) {
+
+        auto resources = withoutDuplicates();
+        std::ranges::sort(resources, std::less{}, &PackageKitResource::nameSortKey);
+
+        for (auto resource : std::as_const(resources)) {
             const auto changelog = resource->changelog();
             if (changelog.isEmpty()) {
                 changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>No release notes provided", resource->packageName(), resource->availableVersion());
@@ -202,7 +206,6 @@ public:
                                 changelog);
             }
         }
-        changes.sort();
 
         if (isDistroUpgrade()) {
             changes.prepend(m_distroUpgrade->description());
@@ -924,5 +927,10 @@ void PackageKitUpdater::setOfflineUpdateAction(PackageKit::Offline::Action actio
     }
 }
 
+bool PackageKitUpdater::isFetchingUpdates() const
+{
+    return m_backend->refresher() != nullptr;
+}
+
 #include "PackageKitUpdater.moc"
 #include "moc_PackageKitUpdater.cpp"
