@@ -8,13 +8,10 @@
 #include "SnapBackend.h"
 #include "SnapResource.h"
 #include "libsnapclient/config-paths.h"
-#include "utils.h"
 #include <KLocalizedString>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QProcess>
-#include <QTimer>
-#include <Snapd/Request>
 
 SnapTransaction::SnapTransaction(QSnapdClient *client, SnapResource *app, Role role, AbstractResource::State newState)
     : Transaction(app, app, role)
@@ -25,7 +22,7 @@ SnapTransaction::SnapTransaction(QSnapdClient *client, SnapResource *app, Role r
     if (role == RemoveRole)
         setRequest(m_client->remove(app->packageName()));
     else if (app->state() == AbstractResource::Upgradeable)
-        setRequest(m_client->refresh(app->packageName()));
+        setRequest(m_client->refresh(app->packageName(), app->m_channel));
     else
         setRequest(m_client->install(app->packageName(), app->m_channel));
 }
@@ -33,6 +30,10 @@ SnapTransaction::SnapTransaction(QSnapdClient *client, SnapResource *app, Role r
 void SnapTransaction::cancel()
 {
     m_request->cancel();
+    if (m_request->error() != QSnapdRequest::NoError) {
+        Q_EMIT passiveMessage(m_request->errorString());
+    }
+    setStatus(CancelledStatus);
 }
 
 void SnapTransaction::finishTransaction()
@@ -46,6 +47,9 @@ void SnapTransaction::finishTransaction()
     case QSnapdRequest::Cancelled:
         setStatus(CancelledStatus);
         break;
+    case QSnapdRequest::AuthDataRequired:
+        setStatus(CancelledStatus);
+        break;
     case QSnapdRequest::NeedsClassic:
         setStatus(SetupStatus);
         if (role() == Transaction::InstallRole) {
@@ -55,32 +59,6 @@ void SnapTransaction::finishTransaction()
             return;
         }
         break;
-    case QSnapdRequest::AuthDataRequired: {
-        setStatus(SetupStatus);
-        QProcess *p = new QProcess;
-        p->setProgram(QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR "/discover/SnapMacaroonDialog"));
-        p->start();
-
-        connect(p, &QProcess::finished, this, [this, p](int code, QProcess::ExitStatus status) {
-            p->deleteLater();
-            if (code != 0) {
-                qWarning() << "login failed... code:" << code << status << p->readAll();
-                Q_EMIT passiveMessage(m_request->errorString());
-                setStatus(DoneWithErrorStatus);
-                return;
-            }
-            const auto doc = QJsonDocument::fromJson(p->readAllStandardOutput());
-            const auto result = doc.object();
-
-            const auto macaroon = result[QStringLiteral("macaroon")].toString();
-            const auto discharges = kTransform<QStringList>(result[QStringLiteral("discharges")].toArray(), [](const QJsonValue &val) {
-                return val.toString();
-            });
-            static_cast<SnapBackend *>(m_app->backend())->client()->setAuthData(new QSnapdAuthData(macaroon, discharges));
-            m_request->runAsync();
-        });
-    }
-        return;
     default:
         qDebug() << "snap error" << m_request.get() << m_request->error() << m_request->errorString();
         Q_EMIT passiveMessage(m_request->errorString());
@@ -100,6 +78,7 @@ void SnapTransaction::setRequest(QSnapdRequest *req)
 
     setCancellable(true);
     connect(m_request.data(), &QSnapdRequest::progress, this, &SnapTransaction::progressed);
+    connect(m_request.data(), &QSnapdRequest::cancel, this, &SnapTransaction::finishTransaction);
     connect(m_request.data(), &QSnapdRequest::complete, this, &SnapTransaction::finishTransaction);
 
     setStatus(CommittingStatus);
@@ -108,22 +87,50 @@ void SnapTransaction::setRequest(QSnapdRequest *req)
 
 void SnapTransaction::progressed()
 {
+    Status status = SetupStatus;
     const auto change = m_request->change();
-    int percentage = 0, count = 0;
+    int percentage = 0, downloadTaskPos = 0;
+    qint64 totalProgressDone = 0, progressTotal = 0;
 
-    auto status = SetupStatus;
     for (int i = 0, c = change->taskCount(); i < c; ++i) {
-        ++count;
         auto task = change->task(i);
         if (task->kind() == QLatin1String("download-snap")) {
-            status = task->status() == QLatin1String("doing") || task->status() == QLatin1String("do") ? DownloadingStatus : CommittingStatus;
-        } else if (task->kind() == QLatin1String("clear-snap")) {
+            downloadTaskPos = i;
+        } else if ((task->status() != QLatin1String("Doing") || task->status() == QLatin1String("Do")) && task->kind() != QLatin1String("download-snap")) {
             status = CommittingStatus;
         }
-        percentage += (100 * task->progressDone()) / task->progressTotal();
+        if (!task->progressLabel().isEmpty()) {
+            totalProgressDone += task->progressDone();
+            progressTotal += task->progressTotal();
+        }
+    }
+    percentage = (progressTotal > 0) ? (100 * totalProgressDone / progressTotal) : 0;
+    setProgress(percentage);
+    QSnapdTask *downloadTask = change->task(downloadTaskPos);
+    if (downloadTask->status() == QLatin1String("Doing")) {
+        status = DownloadingStatus;
+        setSpeed(downloadTask);
     }
-    setProgress(percentage / qMax(count, 1));
     setStatus(status);
 }
 
+void SnapTransaction::setSpeed(QSnapdTask *downloadTask)
+{
+    auto totalDownloadDone = downloadTask->progressDone();
+    if (!m_downloadTimer.isValid()) {
+        m_downloadTimer.start();
+        m_lastProgressDone = totalDownloadDone;
+    } else {
+        qint64 elapsed = m_downloadTimer.elapsed();
+        if (elapsed > 0) {
+            qint64 deltaProgress = totalDownloadDone - m_lastProgressDone;
+            qint64 speed = (deltaProgress * 1000) / elapsed;
+            setDownloadSpeed(speed);
+
+            m_downloadTimer.restart();
+            m_lastProgressDone = totalDownloadDone;
+        }
+    }
+}
+
 #include "moc_SnapTransaction.cpp"
