@@ -14,12 +14,14 @@
 #include <QObject>
 #include <QScopedPointer>
 #include <QSet>
+#include <QSize>
 #include <QStringList>
 #include <QUrl>
 #include <QVector>
 
 #include <ReviewsBackend/Rating.h>
 
+#include "Category/Category.h"
 #include "PackageState.h"
 #include "discovercommon_export.h"
 
@@ -34,16 +36,18 @@ struct Screenshot {
     {
     }
 
-    Screenshot(const QUrl &thumbnail, const QUrl &screenshot, bool isAnimated)
+    Screenshot(const QUrl &thumbnail, const QUrl &screenshot, bool isAnimated, const QSize &thumbnailSize)
         : thumbnail(thumbnail)
         , screenshot(screenshot)
         , isAnimated(isAnimated)
+        , thumbnailSize(thumbnailSize)
     {
     }
 
     QUrl thumbnail;
     QUrl screenshot;
     bool isAnimated = false;
+    QSize thumbnailSize;
 };
 
 using Screenshots = QVector<Screenshot>;
@@ -230,9 +234,9 @@ public:
      */
     virtual Rating rating() const;
 
-    bool categoryMatches(Category *cat);
+    bool categoryMatches(const std::shared_ptr<Category> &cat);
 
-    QSet<Category *> categoryObjects(const QVector<Category *> &cats) const;
+    QSet<std::shared_ptr<Category>> categoryObjects(const QList<std::shared_ptr<Category>> &cats) const;
 
     /**
      * @returns a url that uniquely identifies the application
@@ -326,8 +330,7 @@ Q_SIGNALS:
 private:
     void reportNewState();
 
-    //     TODO: make it std::optional or make QCollatorSortKey()
-    QScopedPointer<QCollatorSortKey> m_collatorKey;
+    std::optional<QCollatorSortKey> m_collatorKey;
     QJsonObject m_metadata;
 };
 
