@@ -27,6 +27,7 @@ class DiscoverNotifier : public QObject
     Q_PROPERTY(State state READ state NOTIFY stateChanged)
     Q_PROPERTY(bool needsReboot READ needsReboot NOTIFY needsRebootChanged)
     Q_PROPERTY(bool busy READ isBusy NOTIFY busyChanged)
+    Q_PROPERTY(bool isSystemUpdateable READ isSystemUpdateable NOTIFY stateChanged)
 public:
     enum State {
         NoUpdates,
@@ -52,6 +53,7 @@ public:
     {
         return m_hasSecurityUpdates;
     }
+    bool isSystemUpdateable() const;
 
     QStringList loadedModules() const;
     bool needsReboot() const
@@ -66,8 +68,9 @@ public:
     }
     UpdatesSettings *settings() const
     {
-        return m_settings;
+        return m_settings.get();
     }
+    void startUnattendedUpdates();
 
 public Q_SLOTS:
     void recheckSystemUpdateNeededAndNotifyApp();
@@ -91,7 +94,9 @@ private:
     void updateStatusNotifier();
     void refreshUnattended();
 
+    bool checkTriggerTimes(const QDateTime &lastTriggerTime) const;
     bool notifyAboutUpdates() const;
+    bool proceedUnattended() const;
 
     QList<BackendNotifierModule *> m_backends;
     QTimer m_timer;
@@ -100,8 +105,8 @@ private:
     bool m_needsReboot = false;
     bool m_isBusy = false;
     QPointer<KNotification> m_updatesAvailableNotification;
-    UnattendedUpdates *m_unattended = nullptr;
+    std::unique_ptr<UnattendedUpdates> m_unattended;
     KConfigWatcher::Ptr m_settingsWatcher;
     QDateTime m_lastUpdate;
-    UpdatesSettings *m_settings;
+    std::unique_ptr<UpdatesSettings> m_settings;
 };
